		title   MZSAVE - Programm im MZ-Format abspeichern

; ===========================================================================
; Programm zum Speichern von Dateien auf Datassette im MZ-Format. 
; Der Aufruf erfolgt durch
;
;   MZSAVE optionen dosfilename "tapefilename"
;
; Hierdurch wird die im MZF-Format vorliegende Datei dosfilename unter dem
; Namen tapefilename auf Cassette im MZ-Format abgespeichert. Alle Parameter
; ausser dosfilename sind optional. Fuer Einzelheiten siehe den Hilfetext.
;
; Autor: Andreas Jung, Karl-Heinz Mau
; Datum: 27.11.1998
; ===========================================================================

main            segment

; ===========================================================================
; Registerbelegung in den kritischen Prozeduren:
; ---------------------------------------------------------------------------
; AX    Akku, Kommunikation mit den Ports
; BL    zu schreibendes Byte
; BH    Bitzaehler
; CX    Zaehler fuer den Countdown und die zu schreibenden Bytes
; DX    Portadresse
; SI    Quelladresse beim Schreiben
; DI    Pruefsumme
; ===========================================================================
		
		assume  cs:main, ds:main, es:main, ss:main

; ===========================================================================
; Konstanten und Portadressen
; ===========================================================================

portb           equ     61h             ; Port B
prndata         equ     0378h           ; Datenregister des Druckerports
prnctrl         equ     0379h           ; Steuerregister des Druckerportes
counter         equ     042h            ; Zaehler
cntctrl         equ     043h            ; Steuerregister des Zaehlers
mark0           equ     199             ; Space-Laenge fuer ein 0-Bit (240s MZ-700)
; for MZ-800: mark0=240s space0=278s mark1=470s space1=494s read point 379s
; read point MZ-700=368s
; clocking of 8253/8254: PC=1.193046 MHz, IBM PC=1.193180 MHz (difference has no meaning)
; Olivetti M24 1.228800 MHz (I don't know if this program can run under M24)
space0          equ     198             ; Mark-Laenge fuer ein 0-Bit (264s MZ-700)
mark1           equ     397             ; Space-Laenge fuer ein 1-Bit (464s MZ-700)
space1          equ     398            ; Mark-Laenge fuer ein 1-Bit (494s MZ-700)

; ===========================================================================
; Macro startcnt startet den Zaehler neu
; ===========================================================================

startcnt        macro
		mov     al, 10111000b   ; 16 Bit laden, Modus 4
; counter 2, LSB gefolgt von MSB
; Modus 4=
; software triggered strobe
; countdown mit output high bis zhler=0, Bei 0 output low fr eine Taktperiode (clk)
; countdown wird durch laden des zhlers getriggert
; neuladen des zhlers wirkt sich auf den nchsten clk puls aus
		out     cntctrl, al     ; Steuerwort schreiben
		mov     al, 0           ; Zaehler an den Anfang setzen
		out     counter, al     ; Lo-Byte schreiben
		out     counter, al     ; Hi-Byte schreiben
		endm

; ===========================================================================
; Macro startcntb startet den Zaehler neu mit ein-Byte-Zaehlung
; ===========================================================================

startcntb       macro
		mov     al, 10011000b   ; 8 Bit laden, Modus 4
		out     cntctrl, al     ; Steuerwort schreiben
		mov     al, 0           ; Zaehler an den Anfang setzen
		out     counter, al     ; Startwert schreiben
		endm

; ===========================================================================
; Macro readcnt liest den aktuellen Zaehlerstand in den Akkumulator ein.
; ===========================================================================

readcnt         macro
		mov     al, 10000000b   ; Zaehler-Latch-Befehl
		out     cntctrl, al     ; Steuerwort schreiben
		in      al, counter     ; Lo-Byte lesen
		mov     ah, al          ; und retten
		in      al, counter     ; Hi-Byte lesen
		xchg    ah, al          ; und merken
		endm

; ===========================================================================
; Macro readcntb liest den aktuellen Zaehlerstand bei ein-Byte-Zaehlung in
; den Akkumulator AL ein.
; ===========================================================================

readcntb        macro
		in      al, counter     ; aktuellen Zaehlerstand einlesen
		endm

; ===========================================================================
; Macro print gibt die $-terminierte Zeichenkette ab der Adresse, die durch
; das Argument gegeben ist, auf dem Bildschirm aus.
; ===========================================================================

print           macro   adr
		mov     ah, 09h         ; DOS-Funktion 09h
		lea     dx, adr         ; Adresse der Zeichenkette
		int     21h             ; Zeichenkette ausgeben
		endm

; ===========================================================================
; Adressen im PSP
; ===========================================================================

		org     80h
arglen          db      ?               ; Laenge der Kommandozeile
args            db      7fh dup (?)     ; Kommandozeile

; ===========================================================================
; Beginn des Programmcodes
; ===========================================================================

		org     100h            ; Start des .COM-Programmes
start:          jmp     mainprog        ; an den Anfang springen

; ===========================================================================
; Diverse Texte
; ===========================================================================

info:
db "Ŀ", 0dh, 0ah
db " MZSAVE V0.2 - MZ700 Saver - (C) 1998 by Andreas Jung and Karl-Heinz Mau ", 0dh, 0ah
db "", 0dh, 0ah
db 0dh, 0ah
db 'Usage: MZSAVE options dosfilename "tapefilename"', 0dh, 0ah
db 0dh, 0ah
db "Saves the file dosfilename, which must be in M12/MZF format, as the file", 0dh, 0ah
db "tapefilename in MZ-format to the datassette. The parameter tapefilename is", 0dh, 0ah
db "optional. If it is omitted, then the name given in the M12/MZF tape header is", 0dh, 0ah
db "used instead.", 0dh, 0ah
db 0dh, 0ah
db "The following options can be specified:", 0dh, 0ah
db 0dh, 0ah
db "        -x        invert polarity when writing", 0dh, 0ah
db "        -s        only write a single copy of header and data", 0dh, 0ah
db 0dh, 0ah
db "The program can be aborted by pressing the STOP button on the datassette.", 0dh, 0ah
db "Run MZSAVE only under plain DOS, not under Windows, because during saving,", 0dh, 0ah
db "all interrupts are disabled.", 0dh, 0ah
db "$"

lderror         db      "Disk load error.", 0dh, 0ah, "$"
toobig          db      "File is too big.", 0dh, 0ah, "$"
nomz            db      "No M12/MZF file.", 0dh, 0ah, "$"
press           db      "Press record & play on tape.", 0dh, 0ah, "$"
ok              db      "ok", 0dh, 0ah, "$"
saving          db      "Saving ", "$"
eol             db      0dh, 0ah, "$"
ready           db      "Ready.", 0dh, 0ah, "$"
panic           db      "Program aborted.", 0dh, 0ah, "$"

; ===========================================================================
; Umwandlungstabelle SHARPSCII -> ASCII
; ===========================================================================

shasctab        db      000h, 001h, 002h, 003h, 004h, 005h, 006h, 007h
		db      008h, 009h, 00ah, 00bh, 00ch, 00dh, 00eh, 00fh
		db      010h, 011h, 012h, 013h, 014h, 015h, 016h, 017h
		db      018h, 019h, 01ah, 01bh, 01ch, 01dh, 01eh, 01fh
		db      " !", '"', "#$%&'()*+,-./"
		db      "0123456789:;<=>?"
		db      "@ABCDEFGHIJKLMNO"
		db      "PQRSTUVWXYZ[\]^_"
		db      "?????????~??????"
		db      "\\//?"
		db      "}//\\\\\//^\\//"
		db      "_#e`~tgh/bxdrpc"
		db      "qazwsui͙kfvj"
		db      "n/m//\ol\y{\"
		db      "ĳ"
		db      "´ڿ"
		db      "?ĳ\/"
		db      "??XO?ۜ?"

; ===========================================================================
; Umwandlungstabelle ASCII -> SHARPSCII
; ===========================================================================

ascshtab        db      000h, 001h, 002h, 003h, 004h, 005h, 006h, 007h
		db      008h, 009h, 00ah, 00bh, 00ch, 00dh, 00eh, 00fh
		db      010h, 011h, 012h, 013h, 014h, 015h, 016h, 017h
		db      018h, 019h, 01ah, 01bh, 01ch, 01dh, 01eh, 01fh
		db      020h, 021h, 022h, 023h, 024h, 025h, 026h, 027h
		db      028h, 029h, 02ah, 02bh, 02ch, 02dh, 02eh, 02fh
		db      030h, 031h, 032h, 033h, 034h, 035h, 036h, 037h
		db      038h, 039h, 03ah, 03bh, 03ch, 03dh, 03eh, 03fh
		db      040h, 041h, 042h, 043h, 044h, 045h, 046h, 047h
		db      048h, 049h, 04ah, 04bh, 04ch, 04dh, 04eh, 04fh
		db      050h, 051h, 052h, 053h, 054h, 055h, 056h, 057h
		db      058h, 059h, 05ah, 05bh, 05ch, 05dh, 05eh, 05fh
		db      093h, 0a1h, 09ah, 09fh, 09ch, 092h, 0aah, 097h
		db      098h, 0a6h, 0afh, 0a9h, 0b8h, 0b3h, 0b0h, 0b7h
		db      09eh, 0a0h, 09dh, 0a4h, 096h, 0a5h, 0abh, 0a3h
		db      09bh, 0bdh, 0a2h, 0beh, 0fdh, 080h, 094h, 05fh
		db      09fh, 0adh, 092h, 0a1h, 0bbh, 0a1h, 0a1h, 09fh
		db      092h, 092h, 092h, 0a6h, 0a6h, 0a6h, 0b9h, 041h
		db      045h, 0bbh, 0b9h, 0b7h, 0bah, 0b7h, 0a5h, 0a5h
		db      0bdh, 0a8h, 0b2h, 09fh, 0fbh, 059h, 050h, 0aah
		db      0a1h, 0a6h, 0b7h, 0a5h, 0b0h, 04eh, 0a1h, 0a1h
		db      03fh, 0d0h, 0ceh, 032h, 034h, 021h, 05fh, 0c6h
		db      070h, 070h, 070h, 0fch, 0d3h, 0d3h, 0d3h, 0ceh
		db      0ceh, 0d3h, 0ach, 0ceh, 0ddh, 0ddh, 0ddh, 0ceh
		db      0cdh, 0d1h, 0d2h, 0cbh, 0e0h, 081h, 0cbh, 0cbh
		db      0cdh, 0d0h, 0d1h, 0d2h, 0cbh, 0a7h, 081h, 0d1h
		db      0d1h, 0d2h, 0d2h, 0cdh, 0cdh, 0d0h, 0d0h, 081h
		db      081h, 0ddh, 0d0h, 0c8h, 0c2h, 0d5h, 0c1h, 0d8h
		db      0a1h, 0aeh, 047h, 0ffh, 053h, 0a4h, 0b3h, 097h
		db      046h, 054h, 04fh, 09ch, 0b7h, 0b7h, 092h, 04dh
		db      03dh, 02bh, 03eh, 03ch, 053h, 053h, 02fh, 03dh
		db      07bh, 02ah, 02ah, 0efh, 0b0h, 032h, 0c8h, 020h

; ===========================================================================
; Prozedur initcnt initialisiert den Zaehler.
; ===========================================================================

initcnt         proc    near
		in      al, portb
		and     al, 11111101b   ; Zaehler vom Lautsprecher abkoppeln
		or      al, 00000001b   ; Zaehler aktivieren
		out     portb, al
		ret
initcnt         endp

; ===========================================================================
; Prozedur waiting wartet 2048*CX Timerzyklen
; ===========================================================================

waiting         proc    near
waitloop:       startcnt                ; Zaehler starten
wcntloop:       readcnt                 ; Zaehler auslesen
		cmp     ax, -2048       ; aktuellen Zaehlerstand vergleichen
		ja      wcntloop        ; bis zum Ablauf warten
		loop    waitloop        ; das ganze CX-mal
		ret
waiting         endp

; ===========================================================================
; Prozedur abort verlaesst das Programm panikartig
; ===========================================================================

abort           proc    near
		sti                     ; Interrupts zulassen
		call    motoroff        ; Motor ausschalten
		print   panic           ; Abbruchmeldung ausgeben
		mov     ax, 4c01h       ; Programm mit Fehler beenden
		int     21h             ; DOS aufrufen
abort           endp

; ===========================================================================
; Prozedur shortpulse schreibt einen kurzen Impuls auf das Band.
; ===========================================================================

shortpulse      proc    near
ssloop:		mov     dx, prndata     ; Datenport fuer die Cassette
		mov	al, mark
		out	dx, al
	        readcnt                 ; Zaehler auslesen
		cmp     ax, -mark0      ; Mark-Zeit abwarten
		ja      ssloop          ; solange noch nicht abgelaufen
		startcnt                ; Zaehler neu starten
		mov     dx, prnctrl     ; zwischendurch RECORD-Taste pruefen
		in      al, dx          ; Status holen
		test    al, 20h         ; falls RECORD immer noch gedrueckt
		jz      srecok          ; dann normal weitermachen
		jmp     abort           ; sonst Programm abbrechen
srecok:         mov     dx, prndata     ; Datenport fuer die Cassette
		mov	al, space
		out	dx, al
smloop:         readcnt                 ; Zaehler auslesen
		cmp     ax, -space0     ; Space-Zeit abwarten
		ja	srecok          ; solange noch nicht abgelaufen
		startcnt                ; Zaehler neu starten
		ret
shortpulse      endp

; ===========================================================================
; Prozedur longpulse schreibt einen langen Impuls auf das Band.
; ===========================================================================

longpulse       proc    near
lsloop:		mov     dx, prndata     ; Datenport fuer die Cassette
		mov	al, mark
		out	dx, al
	        readcnt                 ; Zaehler auslesen
		cmp     ax, -mark1      ; Mark-Zeit abwarten
		ja      lsloop          ; solange noch nicht abgelaufen
		startcnt                ; Zaehler neu starten
		mov     dx, prnctrl     ; zwischendurch RECORD-Taste pruefen
		in      al, dx          ; Status holen
		test    al, 20h         ; falls RECORD immer noch gedrueckt
		jz      lrecok          ; dann normal weitermachen
		jmp     abort           ; sonst Programm abbrechen
lrecok:         mov     dx, prndata     ; Datenport fuer die Cassette
		mov	al, space
		out	dx, al
lmloop:         readcnt                 ; Zaehler auslesen
		cmp     ax, -space1     ; Space-Zeit abwarten
		ja      lrecok          ; solange noch nicht abgelaufen
		startcnt                ; Zaehler neu starten
		ret
longpulse       endp
							   
; ===========================================================================
; Prozedur writebyte schreibt das Byte im Register BL auf das Band und
; aktualisiert die Pruefsumme in DI.
; ===========================================================================

writebyte       proc    near
		call    longpulse       ; langer Impuls zur Einleitung
		mov     bh, 8           ; insgesamt 8 Bit schreiben
wbloop:         rcl     bl, 1           ; naechstes Bit in den Carry schieben
		jc      wbone           ; falls Carry, dann 1 schreiben
		call    shortpulse      ; sonst kurzen Impuls fuer 0 schreiben
		jmp     wbnext          ; und naechstes Bit anvisieren
wbone:          call    longpulse       ; langen Impuls fuer 1 schreiben
		inc     di              ; Pruefsumme erhoehen
wbnext:         dec     bh              ; naechstes Bit anvisieren
		jnz     wbloop          ; und weiterschreiben
		ret
writebyte       endp

; ===========================================================================
; Prozedur writegap schreibt einen Gap mit CX kurzen Impulsen, einen Tape-
; mark mit BH langen und BL kurzen Impulsen und einen abschliessenden langen
; Impuls auf das Band.
; ===========================================================================

writegap        proc    near
wgloop:         call    shortpulse      ; CX kurze Impulse schreiben
		loop    wgloop
		mov     cl, bh          ; BH lange Impulse schreiben
wg1loop:        call    longpulse
		loop    wg1loop
		mov     cl, bl          ; BL kurze Impulse schreiben
wg0loop:        call    shortpulse
		loop    wg0loop
		call    longpulse       ; langen Impuls schreiben
		ret
writegap        endp

; ===========================================================================
; Prozedur writerpt schreibt 256 kurze Impulse auf das Band.
; ===========================================================================

writerpt        proc    near
		mov     cx, 256         ; 256 kurze Impulse
wrloop:         call    shortpulse      ; auf das Band schreiben
		loop    wrloop
		ret
writerpt        endp

; ===========================================================================
; Prozedur writeblk schreibt den Block von CX Bytes ab Adresse SI inklusive
; der dazugehoerigen Pruefsumme und einem abschliessenden langen Impuls auf
; das Band.
; ===========================================================================

writeblk        proc    near
		mov     di, 0           ; Pruefsumme initialisieren
		cmp     cx, 0           ; falls kein Byte zu schreiben
		je      wblkok          ; dann fertig
wblkloop:       mov     bl, [si]        ; Byte holen
		call    writebyte       ; und schreiben
		inc     si              ; naechstes Byte anvisieren
		loop    wblkloop        ; weiterschreiben
wblkok:         mov     cx, di          ; Pruefsumme holen
		mov     bl, ch          ; Hi-Byte der Pruefsumme
		call    writebyte       ; schreiben
		mov     bl, cl          ; Lo-Byte der Pruefsumme
		call    writebyte       ; schreiben
		call    longpulse       ; langen Impuls schreiben
		ret
writeblk        endp

; ===========================================================================
; Prozedur sprint gibt den String ab Adresse SI der Laenge CX auf dem
; Bildschirm aus.
; ===========================================================================

sprint          proc    near
		cmp     cx, 0           ; falls Laenge gleich null
		je      spready         ; dann nichts ausgeben
sploop:         mov     dl, [si]        ; Zeichen holen
		mov     ah, 02h
		int     21h             ; Zeichen ausgeben
		inc     si              ; naechstes Zeichen anvisieren
		loop    sploop          ; und weitermachen
spready:        ret
sprint          endp

; ===========================================================================
; Prozedur shprint gibt den SHARPSCII-String ab Adresse SI der Laenge CX auf 
; dem Bildschirm aus.
; ===========================================================================

shprint         proc    near
		lea     bx, shasctab    ; Uebersetzungstabelle
		cmp     cx, 0           ; falls Laenge gleich null
		je      shpready        ; dann nichts ausgeben
shploop:        mov     al, [si]        ; Zeichen holen
		xlatb                   ; uebersetzen
		mov     dl, al
		mov     ah, 02h
		int     21h             ; Zeichen ausgeben
		inc     si              ; naechstes Zeichen anvisieren
		loop    shploop         ; und weitermachen
shpready:       ret
shprint         endp

; ===========================================================================
; Prozedur prepargs bearbeitet die Kommandozeile. Es werden der Name der 
; DOS-Datei und der Name der gesuchten Cassettendatei sowie gegebenenfalls 
; weitere Kommandozeilenparameter gesucht. Ist die Kommandozeile fehlerhaft,
; so wird ein gesetztes Carry-Flag zurueckgegeben.
; ===========================================================================

prepargs        proc    near
		mov     ax, 0           ; Initialisierungen vornehmen
		mov     dosname, ax     ; Name der DOS-Datei
		mov     dnlen, ax       ; Laenge des Namen
		mov     tapename, ax    ; Name der Cassettendatei
		mov     tnlen, ax       ; Laenge des Namen
		mov     inverse, al     ; Invers-Option
		mov     single, al      ; Single-Option
		mov     space, 82h      ; Schreibwert fuer Space
		mov     mark, 80h       ; Schreibwert fuer Mark
		lea     si, args        ; Kommandozeile absuchen
		mov     dl, arglen      ; Laenge der Kommandozeile
		mov     dh, 0           ; nach DX
		add     dx, si          ; plus Anfang = Ende
paloop:         cmp     si, dx          ; falls nicht Kommandozeilenende
		jb      paget           ; dann weitermachen
		jmp     paready         ; sonst fertig
paget:          mov     al, [si]        ; Zeichen holen
		cmp     al, " "         ; falls Blank
		jne     noblank
		inc     si              ; dann ueberlesen
		jmp     paloop          ; und weitersuchen
noblank:        cmp     al, "-"         ; falls Minuszeichen
		jne     nominus
		inc     si              ; auf erstes Optionszeichen zeigen
		cmp     si, dx          ; falls Kommandozeilenende
		jae     paerror         ; dann Fehler melden
		mov     al, [si]        ; erstes Optionszeichen holen
		cmp     al, " "         ; falls keines vorhanden
		je      paerror         ; dann Fehler melden
optloop:        cmp     al, "x"         ; falls Option -x (invertieren)
		jne     noxopt
		mov     al, inverse     ; falls Option schon angegeben
		cmp     al, 0
		jne     paerror         ; dann Fehler melden
		mov     inverse, 1      ; sonst Option eintragen
		mov     space, 80h      ; inverser Schreibwert fuer Space
		mov     mark, 82h       ; inverser Schreibwert fuer Mark
optnext:        inc     si              ; naechstes Optionszeichen anvisieren
		cmp     si, dx          ; falls Kommandozeilenende
		jb      optget
		jmp     paready         ; dann fertig
optget:         mov     al, [si]        ; sonst Optionszeichen holen
		cmp     al, " "         ; falls Blank
		je      paloop          ; dann naechsten Parameter
		jmp     optloop         ; sonst naechste Option
noxopt:         cmp     al, "s"         ; falls nicht Option -s (single copy)
		jne     paerror         ; dann unerlaubte Option
		mov     al, single      ; falls Option schon angegeben
		cmp     al, 0
		jne     paerror         ; dann Fehler melden
		mov     single, 1       ; sonst Option eintragen
		jmp     optnext         ; und naechst Option bearbeiten
paerror:        stc                     ; bei Fehler Misserfolg melden
		ret
nominus:        cmp     al, '"'         ; falls Anfuehrungszeichen
		jne     noquote
		mov     ax, tapename    ; dann Tape-Namen holen
		cmp     ax, 0           ; falls Tape-Name schon da
		jne     paerror         ; dann Fehler
		inc     si              ; auf erstes Namenzeichen zeigen
		mov     tapename, si    ; Namensanfang ablegen
		mov     cx, 0           ; Zaehler fuer die Namenlaenge
tnloop:         cmp     si, dx          ; falls Kommandozeilenende
		jae     paerror         ; dann Fehler
		mov     al, [si]        ; Zeichen holen
		cmp     al, '"'         ; falls Anfuehrungszeichen
		je      quotstop        ; dann fertig
		inc     cx              ; sonst Laenge erhoehen
		inc     si              ; naechstes Zeichen anvisieren
		jmp     tnloop          ; und weiterlesen
quotstop:       mov     tnlen, cx       ; Laenge des Tape-Namen ablegen
testblank:      inc     si              ; naechstes Zeichen anvisieren
		cmp     si, dx          ; falls Kommandozeilenende
		jae     paready         ; dann fertig
		mov     al, [si]        ; sonst Zeichen holen
		cmp     al, " "         ; es muss ein Blank sein
		jne     paerror         ; wenn nicht, dann Fehler
		jmp     paloop          ; ansonsten weiterlesen
noquote:        mov     ax, dosname     ; DOS-Namen holen
		cmp     ax, 0           ; falls DOS-Name schon da
		jne     paerror         ; dann Fehler
		mov     dosname, si     ; sonst DOS-Name gefunden
		mov     cx, 0           ; Laenge zaehlen
dnloop:         inc     cx              ; Laenge erhoehen
		inc     si              ; naechstes Zeichen anvisieren
		cmp     si, dx          ; falls Kommandozeilenende
		jae     dnstop          ; dann Name fertig
		mov     al, [si]        ; sonst Zeichen holen
		cmp     al, " "         ; falls Blank
		je      dnstop          ; dann Name fertig
		cmp     al, '"'         ; falls Anfuehrungszeichen
		je      paerror         ; dann Fehler
		jmp     dnloop          ; sonst weiterlesen
dnstop:         mov     dnlen, cx       ; Namenlaenge ablegen
		mov     al, 0           ; Namen mit 0-Byte abschliessen
		mov     [si], al
		inc     si              ; und dahinter aufsetzen
		jmp     paloop          ; zur Suchschleife zurueck
paready:        mov     ax, dosname     ; falls kein DOS-Name angegeben
		cmp     ax, 0
		je      paerror         ; dann Fehler
paok:           clc                     ; sonst Erfolg melden
		ret
prepargs        endp

; ===========================================================================
; Prozedur motoron schaltet den Motor ein und wartet die Anlaufzeit ab.
; ===========================================================================

motoron         proc    near
		cli                     ; Interrupts sperren
		mov     dx, prndata     ; Motor einschalten
		mov     al, 80h
		out     dx, al
		call    initcnt         ; Zaehler initialisieren
		startcnt                ; und starten
		mov     cx, 244         ; halbe Sekunde warten
		call    waiting
		sti
		ret
motoron         endp

; ===========================================================================
; Prozedur motoroff schaltet den Motor aus.
; ===========================================================================

motoroff        proc    near
		mov     dx, prndata     ; Motor ausschalten
		mov     al, 00h
		out     dx, al
		ret
motoroff        endp

; ===========================================================================
; Prozedur recordkey ueberprueft den Zustand der RECORD-Taste und fordert
; gegebenenfalls dazu auf, diese zu druecken.
; ===========================================================================

recordkey       proc    near
		mov     dx, prnctrl     ; RECORD-Taste abfragen
		in      al, dx
		test    ax, 20h
		jz      recordok        ; falls gedrueckt, dann ok
		print   press           ; sonst "press ... on tape" ausgeben
		mov     dx, prnctrl     ; RECORD-Taste erneut abfragen
recordloop:     in      al, dx
		test    ax, 20h
		jnz     recordloop      ; solange bis gedrueckt
		print   ok              ; dann "ok" ausgeben
recordok:       call    motoron         ; Motor einschalten
		ret
recordkey       endp

; ===========================================================================
; Prozedur savemsg gibt aus, was gespeichert wird.
; ===========================================================================

savemsg         proc    near
		print   saving          ; "saving " ausgeben
		lea     si, filename    ; Name der zu schreibenden Datei
		mov     cx, fnlen       ; Laenge des Dateinamen
tnamok:         call    shprint         ; Dateinamen ausgeben
		print   eol             ; Zeilenende ausgeben
		ret
savemsg         endp

; ===========================================================================
; Prozedur writefile schreibt die Datei samt Header auf Band.
; ===========================================================================

writefile       proc    near
		cli                     ; Interrupts sperren
		call    initcnt         ; Zaehler initialisieren
		startcnt                ; und starten
		call    shortpulse      ; einen allerersten Impuls schreiben
		mov     cx, 10000       ; langen Gap
		mov     bx, 2828H       ; und langen Tapemark
		call    writegap        ; schreiben
		lea     si, attrib      ; Beginn des Headers
		mov     cx, 128         ; Laenge des Headers
		call    writeblk        ; Header schreiben
		call    writerpt        ; kurze Impulse vor der Wiederholung
		lea     si, attrib      ; Beginn des Headers
		mov     cx, 128         ; Laenge des Headers
		cmp     single, 0       ; falls Single-Option angegeben
		je      wfhdc
		mov     cx, 0           ; dann leere Headerkopie schreiben
wfhdc:          call    writeblk        ; Header schreiben
		mov     cx, 11000       ; kurzen Gap
		mov     bx, 1414H       ; und kurzen Tapemark
		call    writegap        ; schreiben
		lea     si, data        ; Beginn der Daten
		mov     cx, datalen     ; Laenge der Daten
		call    writeblk        ; Daten schreiben
		call    writerpt        ; kurze Impulse vor der Wiederholung
		lea     si, data        ; Beginn der Daten
		mov     cx, datalen     ; Laenge der Daten
		cmp     single, 0       ; falls Single-Option angegeben
		je      wfdatac
		mov     cx, 0           ; dann leere Datenkopie schreiben
wfdatac:        call    writeblk        ; Daten schreiben
		mov     cx, 256         ; Sicherheitsabstand schreiben
wfnloop:        call    shortpulse      ; kurzen Impuls schreiben
		call    longpulse       ; langen Impuls schreiben
		loop    wfnloop         ; und weitermachen
		sti                     ; Interrupts wieder zulassen
		ret
writefile       endp

; ===========================================================================
; Prozedur prephead bereitet den Header zur Speicherung vor. Insbesondere
; wird der Dateiname modifiziert, falls er in der Kommandozeile angegeben
; wurde.
; ===========================================================================

prephead        proc    near
		lea     bx, ascshtab            ; Uebersetzungstabelle
		mov     si, tapename            ; Bandname in Kommandozeile
		cmp     si, 0                   ; falls nicht angegeben
		je      phnotn                  ; dann verzweigen
		lea     di, filename            ; Zieladresse Dateiname
		mov     cx, tnlen               ; Laenge des Dateinamen
		cmp     cx, 16                  ; mit Maximallaenge vergl.
		jbe     phtnlenok               ; falls zu lang:
		mov     cx, 16                  ; dann abschneiden
phtnlenok:      mov     fnlen, cx               ; Namenlaenge ablegen
		cmp     cx, 0                   ; falls Laenge 0
		je      phtrexit                ; dann nichts kopieren
phtrloop:       mov     al, [si]                ; Zeichen holen
		xlatb                           ; uebersetzen
		mov     [di], al                ; und ablegen
		inc     si                      ; naechstes Zeichen anvis.
		inc     di                      ; naechste Zielposition
		loop    phtrloop                ; und weitermachen
phtrexit:       mov     al, 0dh                 ; und mit Carriage Return
		mov     [di], al                ; abschliessen
		mov     al, 00h                 ; Rest mit Nullen fuellen
phtnloop:       inc     di                      ; naechste Position anvis.
		cmp     di, offset datalen      ; falls Bereichsende erreicht
		jae     phtnready               ; dann fertig
		mov     [di], al                ; sonst ablegen
		jmp     phtnloop                ; und weitermachen
phnotn:         mov     al, filename[si]        ; Ende des vorhandenen ...
		cmp     al, 0dh                 ; Dateinamen suchen
		je      phtncr                  ; verzweigen falls CR gefund.
		inc     si                      ; sonst naechste Pos. anvis.
		cmp     si, 17                  ; eventuell Bereichsende ...
		jae     phtncr                  ; als Namenende tolerieren
		jmp     phnotn                  ; sonst weitersuchen
phtncr:         mov     fnlen, si               ; Namenlaenge ablegen
phtnready:      ret
prephead        endp

; ===========================================================================
; Prozedur load laedt das auf Band zu schreibende Programm ein. Beim Auftre-
; ten eines Fehlers wird eine entsprechende Fehlermeldung ausgegeben und das 
; gesetzte Carry-Flag zurueckgegeben.
; ===========================================================================

load            proc    near
		mov     ah, 3dh         ; Datei oeffnen
		mov     al, 00100000b   ; Zugriffsmodus 'nur lesen'
		mov     dx, dosname     ; Dateiname
		int     21h             ; DOS aufrufen
		jnc     ldget           ; falls kein Fehler, dann einladen
lderr:          print   lderror         ; sonst Ladefehler melden
		stc                     ; Fehlerkennung setzen
		ret                     ; Ladeversuch abbrechen
ldget:          mov     bx, ax                    ; Dateihandle
		mov     ah, 3fh                   ; Datei lesen
		mov     cx, sp                    ; verfuegbaren Speicher
		sub     cx, offset attrib+100h    ; ermitteln
		lea     dx, attrib                ; Anfang Ladebereich
		int     21h                       ; DOS aufrufen
		jnc     ldgetok         ; falls Ladefehler aufgetreten
		print   lderror         ; Ladefehler melden
ldcloserr:      mov     ah, 3eh         ; Datei schliessen
		int     21h             ; DOS aufrufen
		stc                     ; Fehlerkennung setzen
		ret                     ; Ladeversuch abbrechen
ldgetok:        cmp     ax, cx          ; falls Anzahl gelesener Bytes
		jb      ldsmall         ; zu gross
		print   toobig          ; dann Fehlermeldung ausgeben
		jmp     ldcloserr       ; und Ladeversuch abbrechen
ldsmall:        cmp     ax, 128         ; falls weniger als 128 Bytes gelesen
		jae     ldbig           ; dann Fehler melden:
ldnomz:         print   nomz            ; keine MZ-Datei
		jmp     ldcloserr       ; Ladeversuch abbrechen
ldbig:          sub     ax, 128         ; Headerlaenge abschneiden
		cmp     ax, datalen     ; Datenlaengenangabe im Header mit Filelaenge vergleichen
;		jne     ldnomz          ; sonst keine MZ-Datei
		jnc	ldmzok		; bei grer/gleich lesen in Datenlaenge des Headers
		mov	datalen,ax	; bei kleiner nur in der Filelaenge lesen
; die folgenden Befehle entfernt zur Anpassung an echte MZ-Verarbeitung,
; trotz der Fehlermoeglichkeit, dass eine MZ-Datei die max. Groesse des Speichers
; ueberschreitet, wenn die Ladeadresse plus Dateilaenge groesser als $FFFF ist. Dies ueberlasse
; ich dem MZ ebenso wie den oben korrigierten Sprung fuer die Faelle bei denen die
; Laengenangabe im Header nicht mit der Dateilaenge uebereinstimmt. Der MZ liest immer nur in
; der Laenge wie im Header vorgegeben, max. bis zur tatsaechlichen Dateilaenge. Das kann er! ;-)
;		mov     cx, loadadr     ; Ladeadresse
;		cmp     cx, 0000h       ; Sonderfall Ladeadresse 0 abfangen
;		je      ldmzok          ; dann ist alles ok
;		neg     cx              ; sonst Ladeadresse negieren (ergibt max. Dateilaenge)
;		cmp     ax, cx          ; falls ax+Ladeadresse > 10000 hex
;		ja      ldnomz          ; dann keine MZ-Datei
ldmzok:         mov     ah, 3eh         ; Datei schliessen
		int     21h             ; DOS aufrufen
		jnc     ldclosok        ; falls Fehler aufgetreten
		print   lderror         ; Ladefehler melden
		stc                     ; Fehlerkennung setzen
ldclosok:       ret                     ; Prozedur beenden
load            endp

; ===========================================================================
; Hauptprogramm
; ===========================================================================

mainprog:       call    prepargs        ; Kommandozeile bearbeiten
		jnc     argsok          ; falls Kommandozeilenfehler
		print   info            ; Informationen ausgeben
errexit:        mov     ax, 4c01h       ; Programm mit Fehler beenden
		int     21h
argsok:         call    load            ; Datei einladen
		jc      errexit         ; im Fehlerfall abbrechen
		call    prephead        ; Header vorbereiten
		call    recordkey       ; RECORD-Taste druecken
		call    savemsg         ; Meldung "saving ..." ausgeben
		call    writefile       ; Programm auf Band schreiben
		print   ready           ; "ready" ausgeben
		call    motoroff        ; Motor ausschalten
		mov     ax, 4c00h       ; Programm erfolgreich beenden
		int     21h

; ===========================================================================
; Uninitialisierter Datenbereich
; ===========================================================================

		align   word            ; allgemeine Variablen

dosname         dw      ?               ; Name der DOS-Datei (Adresse)
dnlen           dw      ?               ; Laenge des Namen
tapename        dw      ?               ; Name der Cassettendatei (Adresse)
tnlen           dw      ?               ; Laenge des Namen
fnlen           dw      ?               ; Laenge des gefundenen Namen
inverse         db      ?               ; <>0 = Polaritaet invertieren
single          db      ?               ; <>0 = keine Kopie schreiben
space           db      ?               ; Schreibwert fuer Space
mark            db      ?               ; Schreibwert fuer Mark

		align   word            ; Header und Daten

attrib          db      ?               ; Dateiattribut
filename        db      17 dup (?)      ; Dateiname
datalen         dw      ?               ; Dateilaenge
loadadr         dw      ?               ; Ladeadresse
startadr        dw      ?               ; Ausfuehrungsadresse
comments        db      104 dup (?)     ; Kommentar
data            db      ?

; ===========================================================================
; Ende des Programmsegmentes
; ===========================================================================

main            ends
		end     start
