(***************************************************************************)
(*                               M o t o r                                 *)
(*=========================================================================*)
(* Motorsteuerung fuer einen an den Druckerport angeschlossenen Daten-     *)
(* recorder. Das Programm schaltet den Motor an, wenn am Recorder eine     *)
(* Bewegungstaste gedrueckt wird, und aus, wenn die STOP-Taste gedrueckt   *)
(* wird. Durch Druecken der ESC-Taste wird das Programm beendet.           *)
(*=========================================================================*)
(* Autor: Andreas Jung                                                     *)
(* Datum: 21.12.1998                                                       *)
(***************************************************************************)

PROGRAM Motor;

  CONST PRNDATA = $0378; (* Datenregister des Druckerports    *)
        PRNCTRL = $0379; (* Kontrollregister des Druckerports *)

  (*************************************************************************)
  (*                          S w i t c h M o t o r                        *)
  (*=======================================================================*)
  (* in:     x     TRUE=ein, FALSE=aus                                     *)
  (*=======================================================================*)
  (* Schaltet den Cassettenmotor entsprechend dem Argument x ein oder aus. *)
  (*************************************************************************)

  PROCEDURE SwitchMotor(x: BOOLEAN);

  BEGIN
    IF x THEN port[PRNDATA] := $80 ELSE port[PRNDATA] := 0;
  END;

  (*************************************************************************)
  (*                           C a s s M o t o r                           *)
  (*=======================================================================*)
  (* return: TRUE falls Motor an, sonst FALSE                              *)
  (*=======================================================================*)
  (* Ermittelt, ob der Cassettenmotor ein- oder ausgeschaltet ist.         *)
  (*************************************************************************)

  FUNCTION CassMotor: BOOLEAN;

  BEGIN
    CassMotor := (port[PRNDATA] AND $80)<>0;
  END;

  (*************************************************************************)
  (*                             C a s s K e y                             *)
  (*=======================================================================*)
  (* return: TRUE falls Taste gedrueckt, sonst FALSE                       *)
  (*=======================================================================*)
  (* Stellt fest, ob eine Taste am Recorder gedrueckt ist.                 *)
  (*************************************************************************)

  FUNCTION CassKey: BOOLEAN;

  BEGIN
    CassKey := (port[PRNCTRL] AND $20)=0;
  END;

  (*************************************************************************)
  (*                           P r i n t S t a t e                         *)
  (*=======================================================================*)
  (* keine Parameter                                                       *)
  (*=======================================================================*)
  (* Gibt den aktuellen Zustand von Cassettenmotor und Tasten aus.         *)
  (*************************************************************************)

  PROCEDURE PrintState;

  BEGIN
    Write(#13, 'Key ');
    IF CassKey THEN Write('pressed    ') ELSE Write('not pressed');
    Write(' / Motor ');
    IF CassMotor THEN Write('on ') ELSE Write('off');
    Write(' ');
  END;

  (*************************************************************************)
  (*                            M a i n L o o p                            *)
  (*=======================================================================*)
  (* keine Parameter                                                       *)
  (*=======================================================================*)
  (* Steuert den Motor solange, bis die Taste ESC gedrueckt wird.          *)
  (*************************************************************************)

  PROCEDURE MainLoop;

    VAR ch: CHAR; (* gedrueckte Taste *)

  BEGIN
    IF CassKey AND NOT CassMotor THEN BEGIN
      PrintState;
      WHILE CassKey DO BEGIN
        IF KeyPressed THEN BEGIN
          Read(kbd, ch);
          IF ch=#27 THEN Exit;
        END;
      END;
    END;
    REPEAT
      IF CassKey THEN BEGIN
        SwitchMotor(TRUE);
        PrintState;
        WHILE CassKey DO BEGIN
          IF KeyPressed THEN BEGIN
            Read(kbd, ch);
            IF ch=#27 THEN Exit;
          END;
        END;
      END ELSE BEGIN
        SwitchMotor(FALSE);
        PrintState;
        WHILE NOT CassKey DO BEGIN
          IF KeyPressed THEN BEGIN
            Read(kbd, ch);
            IF ch=#27 THEN Exit;
          END;
        END;
      END;
    UNTIL FALSE;
  END;

  (*************************************************************************)
  (* Hauptprogramm                                                         *)
  (*************************************************************************)

BEGIN
  LowVideo;
  WriteLn('Ŀ');
  WriteLn(' MOTOR - press ESC to exit ');
  WriteLn('');
  MainLoop;
  WriteLn;
END.